@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: bbs_thread_ui.c,v 1.82 2004/01/14 14:44:30 fuyu Exp $
+ * $Id: bbs_thread_ui.c,v 1.84 2004/01/16 08:01:46 fuyu Exp $
  */
 
 #include "config.h"
@@ -207,6 +207,7 @@ struct _PopupContext
   int x_pos;
   int y_pos;
 
+  gboolean now_sticked;
   gboolean now_pointed;
 };
 
@@ -466,6 +467,7 @@ initialize_popup_context(PopupContext *popup_context,
   popup_context->renderer_context.bold_tag = bold_tag;
   popup_context->renderer_context.tag_table = tag_table;
 
+  /* popup_context->now_sticked = FALSE; */
   /* popup_context->now_pointed = FALSE; */
   parent_window
     = (popup_context->parent_context == NULL
@@ -909,6 +911,8 @@ link_mouse_over_cb(BBSThreadView *view, GdkEventMotion *event,
   if (strncmp(link, "<RES>", 5) == 0)
     return FALSE;
 
+  MAIN_THREAD_EVENT_CALLBACK();
+
   g_strlcpy(tmp_buffer, link, PATH_MAX);
 
   tmp_link = strstr(tmp_buffer, "<FAKE>");
@@ -929,6 +933,8 @@ link_mouse_over_cb(BBSThreadView *view, GdkEventMotion *event,
   if (stack_depth == 1)
     message_id = gtk_statusbar_push(statusbar, link_field_id, tmp_buffer);
 
+  ENTER_MAIN_EVENT_LOOP();
+
   return FALSE;
 }
 
@@ -1179,6 +1185,8 @@ link_mouse_out_cb(BBSThreadView *view, GdkEventAny *event,
   PopupContext *popup_context;
   GtkStatusbar *statusbar = application->statusbar;
 
+  MAIN_THREAD_EVENT_CALLBACK();
+
   popup_context = ensure_popup_context(view, thread, application);
 
   if (message_id != 0)
@@ -1193,6 +1201,8 @@ link_mouse_out_cb(BBSThreadView *view, GdkEventAny *event,
 #endif
   hide_popup(popup_context);
 
+  ENTER_MAIN_EVENT_LOOP();
+
   return FALSE;
 }
 
@@ -1234,6 +1244,8 @@ link_mouse_press_cb(BBSThreadView *view, GdkEventButton *event,
       return TRUE;
     }
 
+  MAIN_THREAD_EVENT_CALLBACK();
+
   tmp_link = strstr(link, "<FAKE>");
   if (tmp_link != NULL)
     link = tmp_link + 6;
@@ -1288,6 +1300,9 @@ link_mouse_press_cb(BBSThreadView *view, GdkEventButton *event,
 			 application->default_open_in_tab
 			 ? event->button == 1 : event->button == 2, FALSE);
     }
+
+  ENTER_MAIN_EVENT_LOOP();
+
   return TRUE;
 }
 
@@ -1470,13 +1485,14 @@ bookmark_clicked_cb(GtkWidget *widget, BBSThreadView *view)
 static void
 bookmark_destroy_cb(GtkWidget *widget, BBSThreadGUIInfo *info)
 {
-  CHECK_THREADS_ENTERED();
-
+  /* MAIN_THREAD_EVENT_CALLBACK(); */
+  /* Ĵ٤̵ */
   if (info->bookmark == widget)
     info->bookmark = NULL;
 #if 0
   fprintf(stderr, "bookmark(%p) destroyed.\n", widget);
 #endif
+  /* ENTER_MAIN_EVENT_LOOP(); */
 }
 
 
@@ -2843,39 +2859,61 @@ popup_realize_cb(GtkWidget *widget, PopupContext *popup_context)
 
 
 static gboolean
-popup_window_enter_notify_cb(GtkWidget *widget,
-			     GdkEventCrossing *event,
-			     PopupContext *popup_context)
+popup_window_button_press_event_cb(GtkWidget *widget, GdkEventButton *event,
+				   PopupContext *popup_context)
 {
-#if DEBUG_POPUP
-  fprintf(stderr, "popup_window_enter_notify_cb: widget=%p popup_context=%p\n",
-	  widget, popup_context);
+  popup_context->now_sticked = TRUE;
+  return FALSE;
+}
+
+
+static gboolean
+popup_window_button_release_event_cb(GtkWidget *widget, GdkEventButton *event,
+				     PopupContext *popup_context)
+{
+  popup_context->now_sticked = FALSE;
+  return FALSE;
+}
+
+
+static gboolean
+popup_window_enter_notify_event_cb(GtkWidget *widget,
+				   GdkEventCrossing *event,
+				   PopupContext *popup_context)
+{
+#if 0	/* ̵̤ʤΤ */
+  MAIN_THREAD_EVENT_CALLBACK();
 #endif
+
   if (popup_context->close_delay_id != 0)
     {
       g_source_remove(popup_context->close_delay_id);
       popup_context->close_delay_id = 0;
     }
 
+  popup_context->now_sticked = FALSE;
   popup_context->now_pointed = TRUE;
 
+#if 0	/* ̵̤ʤΤ */
+  ENTER_MAIN_EVENT_LOOP();
+#endif
+
   return FALSE;
 }
 
 
 static gboolean
-popup_window_leave_notify_cb(GtkWidget *widget,
-			     GdkEventCrossing *event,
-			     PopupContext *popup_context)
+popup_window_leave_notify_event_cb(GtkWidget *widget,
+				   GdkEventCrossing *event,
+				   PopupContext *popup_context)
 {
-#if DEBUG_POPUP
-  fprintf(stderr, "popup_window_leave_notify_cb: widget=%p popup_context=%p\n",
-	  widget, popup_context);
-#endif
-  popup_context->now_pointed = FALSE;
+  MAIN_THREAD_EVENT_CALLBACK();
 
+  popup_context->now_pointed = FALSE;
   hide_popup(popup_context);
 
+  ENTER_MAIN_EVENT_LOOP();
+
   return FALSE;
 }
 
@@ -2908,11 +2946,17 @@ render_popup_response(PopupContext *popup_context,
   g_signal_connect(G_OBJECT(view), "link_mouse_release",
 		   G_CALLBACK(link_mouse_release_cb), application);
 
-  g_signal_connect(G_OBJECT(view), "view_enter_notify",
-		   G_CALLBACK(popup_window_enter_notify_cb),
+  g_signal_connect(G_OBJECT(view), "button_press_event",
+		   G_CALLBACK(popup_window_button_press_event_cb),
 		   popup_context);
-  g_signal_connect(G_OBJECT(view), "view_leave_notify",
-		   G_CALLBACK(popup_window_leave_notify_cb),
+  g_signal_connect(G_OBJECT(view), "button_release_event",
+		   G_CALLBACK(popup_window_button_release_event_cb),
+		   popup_context);
+  g_signal_connect(G_OBJECT(view), "enter_notify_event",
+		   G_CALLBACK(popup_window_enter_notify_event_cb),
+		   popup_context);
+  g_signal_connect(G_OBJECT(view), "leave_notify_event",
+		   G_CALLBACK(popup_window_leave_notify_event_cb),
 		   popup_context);
 
   g_signal_connect_after(G_OBJECT(view), "expose_event",
@@ -3215,7 +3259,7 @@ popup_close_delay_timeout(PopupContext *popup_context)
 
   OCHU_THREADS_ENTER();
 
-  if (!popup_context->now_pointed
+  if (!popup_context->now_pointed && !popup_context->now_sticked
       && !(popup_context->renderer_context.view != NULL
 	   && (child_context = g_object_get_qdata(G_OBJECT(popup_context->renderer_context.view), popup_context_id)) != NULL
 	   && child_context->renderer_context.view != NULL))
