@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: bbs_thread_view.c,v 1.41 2004/01/14 11:07:02 fuyu Exp $
+ * $Id: bbs_thread_view.c,v 1.42 2004/01/15 09:31:08 fuyu Exp $
  */
 
 #include "config.h"
@@ -68,24 +68,15 @@ static gboolean text_view_get_iter_at_link_tag_from_event(
 						GtkTextView *text_view,
 						GtkTextIter *iter,
 						GdkEventAny *event);
-static gboolean enter_notify_event_cb(GtkWidget *widget,
-				      GdkEventCrossing *event,
-				      BBSThreadView *view);
-static gboolean leave_notify_event_cb(GtkWidget *widget,
-				      GdkEventCrossing *event,
-				      BBSThreadView *view);
-static gboolean motion_notify_event_cb(GtkTextView *text_view,
-				       GdkEventMotion *event,
-				       BBSThreadView *bbs_thread_view);
+static gboolean bbs_thread_view_motion_notify(GtkWidget *widget,
+					      GdkEventMotion *event);
+static gboolean bbs_thread_view_button_press(GtkWidget *widget,
+					     GdkEventButton *event);
+static gboolean bbs_thread_view_button_release(GtkWidget *widget,
+					       GdkEventButton *event);
 static void emit_link_mouse_over(BBSThreadView *view, GdkEventMotion *event);
 static void emit_link_mouse_out(BBSThreadView *view, GdkEventAny *event);
 
-static gboolean button_press_event_cb(GtkTextView *text_view,
-				      GdkEventButton *event,
-				      BBSThreadView *bbs_thread_view);
-static gboolean button_release_event_cb(GtkTextView *text_view,
-					GdkEventButton *event,
-					BBSThreadView *bbs_thread_view);
 static void bbs_thread_view_scroll_to(BBSThreadView *view, int to_where);
 
 
@@ -122,8 +113,6 @@ enum {
   LINK_MOUSE_OUT_SIGNAL,
   LINK_MOUSE_PRESS_SIGNAL,
   LINK_MOUSE_RELEASE_SIGNAL,
-  VIEW_ENTER_NOTIFY_SIGNAL,
-  VIEW_LEAVE_NOTIFY_SIGNAL,
   SCROLL_TO_SIGNAL,
   SCROLL_VIEW_SIGNAL,
   INTERACTIVE_SEARCH_SIGNAL,
@@ -141,7 +130,7 @@ enum {
 
 static GtkTextViewClass *parent_class = NULL;
 static int bbs_thread_view_signals[LAST_SIGNAL] =
-  { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
+  { 0, 0, 0, 0, 0, 0, 0, 0, 0, };
 static GtkTextTagTable *default_tag_table;
 
 static GtkTextTag *link_tags[2];
@@ -190,6 +179,7 @@ bbs_thread_view_class_init(BBSThreadViewClass *klass)
 {
   GObjectClass *o_class = (GObjectClass *)klass;
   GtkObjectClass *object_class = (GtkObjectClass *)klass;
+  GtkWidgetClass *widget_class = (GtkWidgetClass *)klass;
   GtkBindingSet *binding_set;
 
   GtkTextTag *tag;
@@ -350,25 +340,6 @@ bbs_thread_view_class_init(BBSThreadViewClass *klass)
 		 G_TYPE_POINTER,
 		 G_TYPE_STRING);
 
-  bbs_thread_view_signals[VIEW_ENTER_NOTIFY_SIGNAL] =
-    g_signal_new("view_enter_notify",
-		 G_TYPE_FROM_CLASS(klass),
-		 G_SIGNAL_RUN_LAST,
-		 G_STRUCT_OFFSET(BBSThreadViewClass, view_enter_notify),
-		 NULL, NULL,
-		 ochusha_marshal_VOID__BOXED,
-		 G_TYPE_NONE, 1,
-		 GDK_TYPE_EVENT | G_SIGNAL_TYPE_STATIC_SCOPE);
-  bbs_thread_view_signals[VIEW_LEAVE_NOTIFY_SIGNAL] =
-    g_signal_new("view_leave_notify",
-		 G_TYPE_FROM_CLASS(klass),
-		 G_SIGNAL_RUN_LAST,
-		 G_STRUCT_OFFSET(BBSThreadViewClass, view_leave_notify),
-		 NULL, NULL,
-		 ochusha_marshal_VOID__BOXED,
-		 G_TYPE_NONE, 1,
-		 GDK_TYPE_EVENT | G_SIGNAL_TYPE_STATIC_SCOPE);
-
   bbs_thread_view_signals[SCROLL_TO_SIGNAL] =
     g_signal_new("scroll_to",
 		 G_TYPE_FROM_CLASS(klass),
@@ -464,6 +435,10 @@ bbs_thread_view_class_init(BBSThreadViewClass *klass)
   gtk_binding_entry_add_signal(binding_set, GDK_W, GDK_CONTROL_MASK,
 			       "close_thread", 0);
 
+  widget_class->motion_notify_event = bbs_thread_view_motion_notify;
+  widget_class->button_press_event = bbs_thread_view_button_press;
+  widget_class->button_release_event = bbs_thread_view_button_release;
+
   klass->link_mouse_over = NULL;
   klass->link_mouse_out = NULL;
   klass->link_mouse_press = NULL;
@@ -501,16 +476,6 @@ bbs_thread_view_init(BBSThreadView *view)
 
   g_signal_connect(G_OBJECT(widget), "realize",
 		   G_CALLBACK(text_view_realize_cb), view);
-  g_signal_connect(G_OBJECT(widget), "enter_notify_event",
-		   G_CALLBACK(enter_notify_event_cb), view);
-  g_signal_connect(G_OBJECT(widget), "leave_notify_event",
-		   G_CALLBACK(leave_notify_event_cb), view);
-  g_signal_connect(G_OBJECT(widget), "motion_notify_event",
-		   G_CALLBACK(motion_notify_event_cb), view);
-  g_signal_connect(G_OBJECT(widget), "button_press_event",
-		   G_CALLBACK(button_press_event_cb), view);
-  g_signal_connect(G_OBJECT(widget), "button_release_event",
-		   G_CALLBACK(button_release_event_cb), view);
 
   gtk_text_buffer_get_start_iter(buffer, &view->iter);
 
@@ -720,63 +685,18 @@ text_view_get_iter_at_link_tag_from_event(GtkTextView *text_view,
 
 
 static gboolean
-enter_notify_event_cb(GtkWidget *widget, GdkEventCrossing *event,
-		      BBSThreadView *view)
-{
-  g_return_val_if_fail(IS_BBS_THREAD_VIEW(view), FALSE);
-
-  MAIN_THREAD_EVENT_CALLBACK();
-
-#if DEBUG_WIDGET_MOST
-  fprintf(stderr, "bbs_thread_view.c: enter_notify_event_cb\n");
-#endif
-
-  g_signal_emit(G_OBJECT(view),
-		bbs_thread_view_signals[VIEW_ENTER_NOTIFY_SIGNAL],
-		0,
-		event);
-
-  ENTER_MAIN_EVENT_LOOP();
-
-  return FALSE;
-}
-
-
-static gboolean
-leave_notify_event_cb(GtkWidget *widget, GdkEventCrossing *event,
-		      BBSThreadView *view)
-{
-  g_return_val_if_fail(IS_BBS_THREAD_VIEW(view), FALSE);
-
-  MAIN_THREAD_EVENT_CALLBACK();
-
-#if DEBUG_WIDGET_MOST
-  fprintf(stderr, "bbs_thread_view.c: leave_notify_event_cb\n");
-#endif
-  if (view->last_offset != -1)
-    emit_link_mouse_out(view, (GdkEventAny *)event);
-
-  g_signal_emit(G_OBJECT(view),
-		bbs_thread_view_signals[VIEW_LEAVE_NOTIFY_SIGNAL],
-		0,
-		event);
-
-  ENTER_MAIN_EVENT_LOOP();
-
-  return FALSE;
-}
-
-
-static gboolean
-motion_notify_event_cb(GtkTextView *text_view, GdkEventMotion *event,
-		       BBSThreadView *bbs_thread_view)
+bbs_thread_view_motion_notify(GtkWidget *widget, GdkEventMotion *event)
 {
+  gboolean result = FALSE;
   GtkTextIter iter;
+  GtkTextView *text_view;
+  BBSThreadView *bbs_thread_view;
 
-  g_return_val_if_fail(GTK_IS_TEXT_VIEW(text_view)
+  g_return_val_if_fail(IS_BBS_THREAD_VIEW(widget)
 		       && event->type == GDK_MOTION_NOTIFY, FALSE);
 
-  MAIN_THREAD_EVENT_CALLBACK();
+  text_view = GTK_TEXT_VIEW(widget);
+  bbs_thread_view = BBS_THREAD_VIEW(widget);
 
   if (text_view_get_iter_at_link_tag_from_event(text_view, &iter,
 						(GdkEventAny *)event))
@@ -784,8 +704,7 @@ motion_notify_event_cb(GtkTextView *text_view, GdkEventMotion *event,
       int start_offset = gtk_text_iter_get_offset(&iter);
       if (start_offset == bbs_thread_view->last_offset)
 	{
-	  ENTER_MAIN_EVENT_LOOP();
-	  return FALSE;
+	  goto finished;
 	}
 
       if (bbs_thread_view->last_offset != -1)
@@ -797,9 +716,12 @@ motion_notify_event_cb(GtkTextView *text_view, GdkEventMotion *event,
   else if (bbs_thread_view->last_offset != -1)
     emit_link_mouse_out(bbs_thread_view, (GdkEventAny *)event);
 
-  ENTER_MAIN_EVENT_LOOP();
+ finished:
+  if (GTK_WIDGET_CLASS(parent_class)->motion_notify_event != NULL)
+    result = (*GTK_WIDGET_CLASS(parent_class)->motion_notify_event)(widget,
+								    event);
 
-  return FALSE;
+  return result;
 }
 
 
@@ -864,77 +786,80 @@ emit_link_mouse_out(BBSThreadView *view, GdkEventAny *event)
 
 
 static gboolean
-button_press_event_cb(GtkTextView *text_view, GdkEventButton *event,
-		      BBSThreadView *bbs_thread_view)
+bbs_thread_view_button_press(GtkWidget *widget, GdkEventButton *event)
 {
+  GtkTextView *text_view;
   GtkTextIter iter;
+  gboolean result = FALSE;
 
-  g_return_val_if_fail(GTK_IS_TEXT_VIEW(text_view)
+  g_return_val_if_fail(IS_BBS_THREAD_VIEW(widget)
 		       && (event->type == GDK_BUTTON_PRESS
 			   || event->type == GDK_2BUTTON_PRESS
 			   || event->type == GDK_3BUTTON_PRESS), FALSE);
+  text_view = GTK_TEXT_VIEW(widget);
 
   if (text_view_get_iter_at_link_tag_from_event(text_view, &iter,
 						(GdkEventAny *)event))
     {
-      gboolean handled;
+      BBSThreadView *bbs_thread_view = BBS_THREAD_VIEW(text_view);
       int start_offset = gtk_text_iter_get_offset(&iter);
       gchar *link = (bbs_thread_view->link_table != NULL)
 	? (gchar *)g_hash_table_lookup(bbs_thread_view->link_table,
 				       (gconstpointer)start_offset)
 	: NULL;
 
-      MAIN_THREAD_EVENT_CALLBACK();
-
       g_signal_emit(G_OBJECT(bbs_thread_view),
 		    bbs_thread_view_signals[LINK_MOUSE_PRESS_SIGNAL],
 		    0,
 		    event,
 		    bbs_thread_view->thread,
 		    link,
-		    &handled);
-
-      ENTER_MAIN_EVENT_LOOP();
-      return TRUE;
+		    &result);
     }
 
-  return FALSE;
+  if (!result && GTK_WIDGET_CLASS(parent_class)->button_press_event != NULL)
+    result = (*GTK_WIDGET_CLASS(parent_class)->button_press_event)(widget,
+								   event);
+
+  return result;
 }
 
 
 static gboolean
-button_release_event_cb(GtkTextView *text_view, GdkEventButton *event,
-			BBSThreadView *bbs_thread_view)
+bbs_thread_view_button_release(GtkWidget *widget, GdkEventButton *event)
 {
+  GtkTextView *text_view;
   GtkTextIter iter;
+  gboolean result = FALSE;
 
-  g_return_val_if_fail(GTK_IS_TEXT_VIEW(text_view)
+  g_return_val_if_fail(GTK_IS_TEXT_VIEW(widget)
 		       && event->type == GDK_BUTTON_RELEASE, FALSE);
+  text_view = GTK_TEXT_VIEW(widget);
 
   if (text_view_get_iter_at_link_tag_from_event(text_view, &iter,
 						(GdkEventAny *)event))
     {
-      gboolean handled;
+      BBSThreadView *bbs_thread_view = BBS_THREAD_VIEW(text_view);
       int start_offset = gtk_text_iter_get_offset(&iter);
       gchar *link = (bbs_thread_view->link_table != NULL)
 	? (gchar *)g_hash_table_lookup(bbs_thread_view->link_table,
 				       (gconstpointer)start_offset)
 	: NULL;
 
-      MAIN_THREAD_EVENT_CALLBACK();
-
       g_signal_emit(G_OBJECT(bbs_thread_view),
 		    bbs_thread_view_signals[LINK_MOUSE_RELEASE_SIGNAL],
 		    0,
 		    event,
 		    bbs_thread_view->thread,
 		    link,
-		    &handled);
-
-      ENTER_MAIN_EVENT_LOOP();
+		    &result);
     }
 
-  return FALSE;
+  if (GTK_WIDGET_CLASS(parent_class)->button_release_event != NULL)
+    result = (*GTK_WIDGET_CLASS(parent_class)->button_release_event)(widget,
+								     event);
+
+  return result;
 }
 
 
