@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: boardlist_ui.c,v 1.75 2004/01/14 23:52:25 fuyu Exp $
+ * $Id: boardlist_ui.c,v 1.78 2004/01/15 15:06:21 fuyu Exp $
  */
 
 #include "config.h"
@@ -1101,6 +1101,8 @@ item_view_being_closed_cb(PanedNotebook *paned_notebook,
   BulletinBoardGUIInfo *info
     = ensure_bulletin_board_info(board, application);
 
+  MAIN_THREAD_EVENT_CALLBACK();
+
   /*
    * MEMO: ɽƤ륦åȤºݤ˼ΤƤ륿ߥ
    *       򤺤餹
@@ -1159,7 +1161,7 @@ item_view_being_closed_cb(PanedNotebook *paned_notebook,
       info->message_id = 0;
     }
 
-  return;
+  ENTER_MAIN_EVENT_LOOP();
 }
 
 
@@ -1882,9 +1884,11 @@ new_thread_dialog_response_cb(GtkWidget *dialog, int response_id,
 					  new_thread_netstat_id,
 					  statusbar_message);
 
+	  OCHU_THREADS_LEAVE();
 	  result = ochusha_bulletin_board_create_new_thread(board,
 						application->broker,
 						title, &response);
+	  OCHU_THREADS_ENTER();
 	  g_free(message);
 	  if (result)
 	    {
