@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: bulletin_board_ui.c,v 1.122 2004/01/15 03:27:26 fuyu Exp $
+ * $Id: bulletin_board_ui.c,v 1.124 2004/01/15 15:04:10 fuyu Exp $
  */
 
 #include "config.h"
@@ -2144,6 +2144,18 @@ thread_title_popup_paint_window(GtkWidget *widget)
 }
 
 
+static void
+close_thread_cb(BBSThreadView *view, PanedNotebook *notebook)
+{
+  GtkWidget *scrolled_window;
+  g_return_if_fail(GTK_IS_WIDGET(view));
+  g_return_if_fail(IS_PANED_NOTEBOOK(notebook));
+  scrolled_window = GTK_WIDGET(view)->parent;
+  g_return_if_fail(GTK_IS_WIDGET(scrolled_window));
+  paned_notebook_close_view(notebook, scrolled_window);
+}
+
+
 static void
 item_view_required_cb(PanedNotebook *paned_notebook,
 		      OchushaBBSThread *thread, GtkWidget **item_view,
@@ -2168,9 +2180,8 @@ item_view_required_cb(PanedNotebook *paned_notebook,
   thread_view = open_bbs_thread(application, thread, ICON_LABEL(*tab_label));
   if (thread_view != NULL)
     {
-      g_signal_connect_swapped(G_OBJECT(thread_view), "close_thread",
-			       G_CALLBACK(paned_notebook_close_current_item_view),
-			       paned_notebook);
+      g_signal_connect(G_OBJECT(thread_view), "close_thread",
+		       G_CALLBACK(close_thread_cb), paned_notebook);
       gtk_widget_show(thread_view);
       *item_view = gtk_scrolled_window_new(NULL, NULL);
       gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(*item_view),
@@ -2226,6 +2237,8 @@ item_view_being_closed_cb(PanedNotebook *paned_notebook,
 
   g_return_if_fail(GTK_IS_WIDGET(item_view));
 
+  MAIN_THREAD_EVENT_CALLBACK();
+
   info = ensure_bbs_thread_info(thread);
   info->recent_view = NULL;
   buffer = info->response_source_buffer;
@@ -2327,6 +2340,8 @@ item_view_being_closed_cb(PanedNotebook *paned_notebook,
 			   info->message_id);
       info->message_id = 0;
     }
+
+  ENTER_MAIN_EVENT_LOOP();
 }
 
 
