@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_ui.c,v 1.85 2004/01/15 03:21:12 fuyu Exp $
+ * $Id: ochusha_ui.c,v 1.87 2004/01/16 19:32:11 fuyu Exp $
  */
 
 #include "config.h"
@@ -1031,7 +1031,8 @@ on_quit(gpointer data, guint action, GtkWidget *widget)
   OCHU_THREADS_LEAVE();
   ochusha_async_buffer_suspend_all();
   ochusha_async_buffer_terminate_all();
-  OCHU_THREADS_ENTER();
+  /* OCHU_THREADS_ENTER(); */
+  gdk_threads_enter();
 
   if (application->sm_client_id != NULL)
     {
@@ -1893,6 +1894,9 @@ access_failed_cb(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer,
 }
 
 
+static gboolean enable_debug_outputs = FALSE;
+
+
 static void
 output_log_cb(OchushaNetworkBroker *broker, const gchar *message,
 	      OchushaApplication *application)
@@ -1903,11 +1907,14 @@ output_log_cb(OchushaNetworkBroker *broker, const gchar *message,
 #if ENABLE_DEBUG_CONSOLE
   g_return_if_fail(GTK_IS_TEXT_BUFFER(log_buffer));
 
-  OCHU_THREADS_ENTER();
+  if (enable_debug_outputs)
+    {
+      OCHU_THREADS_ENTER();
 
-  gtk_text_buffer_insert(log_buffer, &log_tail_iter, message, -1);
+      gtk_text_buffer_insert(log_buffer, &log_tail_iter, message, -1);
 
-  OCHU_THREADS_LEAVE();
+      OCHU_THREADS_LEAVE();
+    }
 #endif
 }
 
@@ -2840,12 +2847,21 @@ debug_console_response_cb(GtkWidget *widget, int response_id, gpointer unused)
 }
 
 
+static void
+display_debug_output_toggled_cb(GtkToggleButton *button,
+				OchushaApplication *application)
+{
+  enable_debug_outputs = gtk_toggle_button_get_active(button);
+}
+
+
 static void
 open_debug_console(OchushaApplication *application)
 {
   GtkWidget *dialog;
   GtkWidget *scroll;
   GtkWidget *text_view;
+  GtkWidget *button;
 
   if (debug_console != NULL)
     {
@@ -2876,7 +2892,15 @@ open_debug_console(OchushaApplication *application)
   gtk_widget_show(text_view);
   gtk_container_add(GTK_CONTAINER(scroll), text_view);
   gtk_widget_show(scroll);
-  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), scroll);
+  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), scroll,
+		     TRUE, TRUE, 0);
+
+  button = gtk_check_button_new_with_label(_("Display Debug Outputs in This Console."));
+  gtk_widget_show(button);
+  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), button,
+		     FALSE, FALSE, 0);
+  g_signal_connect(G_OBJECT(button), "toggled",
+		   G_CALLBACK(display_debug_output_toggled_cb), application);
 
   gtk_widget_show(dialog);
   debug_console = dialog;
