@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: text_search_window.c,v 1.13 2004/01/15 02:18:05 fuyu Exp $
+ * $Id: text_search_window.c,v 1.14 2004/01/15 09:31:32 fuyu Exp $
  */
 
 #include "config.h"
@@ -194,6 +194,9 @@ text_search_window_key_press_event_cb(TextSearchWindow *window,
       return TRUE;
     }
 
+  if (event->keyval == GDK_Return)
+    text_search_window_process_find_next_response(window);
+
   return FALSE;
 }
 
