@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003 The Ochusha Project.
+ * Copyright (c) 2003-2004 The Ochusha Project.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: virtual_board.c,v 1.11 2004/01/13 23:07:08 fuyu Exp $
+ * $Id: virtual_board.c,v 1.15 2004/01/15 02:05:08 fuyu Exp $
  */
 
 #include "config.h"
@@ -201,6 +201,8 @@ virtual_board_remove_thread(VirtualBoard *virtual_board,
   OchushaBulletinBoard *board = (OchushaBulletinBoard *)virtual_board;
   g_return_if_fail(IS_VIRTUAL_BOARD(virtual_board) && IS_THREAD_PROXY(thread));
 
+  ochusha_bulletin_board_lock_thread_list(board);
+
   if (g_hash_table_remove(board->thread_table, thread->id))
     {
       board->thread_list = g_slist_remove(board->thread_list, thread);
@@ -210,6 +212,8 @@ virtual_board_remove_thread(VirtualBoard *virtual_board,
       virtual_board->board_list = NULL;
       g_slist_foreach(board->thread_list, update_board_list, virtual_board);
     }
+
+  ochusha_bulletin_board_unlock_thread_list(board);
 }
 
 
@@ -282,6 +286,8 @@ virtual_board_clean_up_threadlist(VirtualBoard *virtual_board,
 
   board = OCHUSHA_BULLETIN_BOARD(virtual_board);
 
+  ochusha_bulletin_board_lock_thread_list(board);
+
   if (remove_unread_thread)
     {
       GSList *removed_list = NULL;
@@ -304,6 +310,8 @@ virtual_board_clean_up_threadlist(VirtualBoard *virtual_board,
       g_slist_free(virtual_board->recent_thread_list);
       virtual_board->recent_thread_list = NULL;
     }
+
+  ochusha_bulletin_board_unlock_thread_list(board);
 }
 
 
@@ -448,7 +456,7 @@ internal_each_thread_cb(OchushaBBSThread *thread, gpointer user_data)
   if (proxy->real_thread == NULL)
     {
       /* bookmark줿ĤξΤߤ */
-      OCHU_OBJECT_REF(G_OBJECT(thread));
+      OCHU_OBJECT_REF(thread);
       proxy->real_thread = thread;
     }
 
@@ -472,15 +480,16 @@ update_by_board(gpointer list_data, gpointer user_data)
   if (args->terminate_now)
     return;
 
+  OCHU_THREADS_ENTER();
   buffer = ochusha_bulletin_board_get_threadlist_source(board, args->broker,
 							args->buffer,
 							args->mode);
-
   if (board->thread_list == NULL)
     ochusha_bulletin_board_read_threadlist_xml(board,
 					       &application->config,
 					       application->session_subdir,
 					       TRUE);
+  OCHU_THREADS_LEAVE();
 
   if (!ochusha_bulletin_board_refresh_threadlist(OCHUSHA_BULLETIN_BOARD(list_data),
 						 buffer,
