@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003 The Ochusha Project.
+ * Copyright (c) 2003-2004 The Ochusha Project.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: htmlutils.c,v 1.10 2003/12/30 17:22:07 fuyu Exp $
+ * $Id: htmlutils.c,v 1.12 2004/01/14 09:59:48 fuyu Exp $
  */
 
 #include "config.h"
@@ -43,8 +43,6 @@
 #include <string.h>
 
 
-static const char *mempbrk(const char *string, const char *charset,
-			   size_t len);
 static const char *process_tag(ElementHandler *handler, void *context,
 			       const char *head, int len);
 static const char *process_entity_ref(ElementHandler *handler, void *context,
@@ -1059,31 +1057,6 @@ fake_anchor_tag(ElementHandler *handler, void *context, const char *head,
 #endif
 
 
-/*
- * Ĺǽstrpbrkؿ
- * stringĹlenʸcharset˴ޤޤǽʸؤΥݥ󥿤֤
- * ʤNULL֤
- */
-static const char *
-mempbrk(const char *string, const char *charset, size_t len)
-{
-  while (len > 0)
-    {
-      const char *c = charset;
-      char s = *string;
-      while (*c)
-	{
-	  if (*c == s)
-	    return string;
-	  c++;
-	}
-      string++;
-      len--;
-    }
-  return NULL;
-}
-
-
 static gchar *
 encode_unicode_to_utf8(unsigned int character, gchar *buffer)
 {
