@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003 The Ochusha Project.
+ * Copyright (c) 2003-2004 The Ochusha Project.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_bbs_thread.c,v 1.25 2004/01/11 03:04:03 fuyu Exp $
+ * $Id: ochusha_bbs_thread.c,v 1.26 2004/01/14 09:59:48 fuyu Exp $
  */
 
 #include "config.h"
@@ -310,10 +310,10 @@ ochusha_bbs_thread_set_board(OchushaBBSThread *thread,
 
 #if 0	/* Ȥ롼פƤޤΤǱѤǥС */
   if (thread->board != NULL)
-    OCHU_OBJECT_UNREF(G_OBJECT(thread->board));
+    OCHU_OBJECT_UNREF(thread->board);
 
   if (board != NULL)
-    OCHU_OBJECT_REF(G_OBJECT(board));
+    OCHU_OBJECT_REF(board);
 #endif
 
   thread->board = board;
