@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003 The Ochusha Project.
+ * Copyright (c) 2003-2004 The Ochusha Project.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_bulletin_board.c,v 1.29 2004/01/13 14:52:19 fuyu Exp $
+ * $Id: ochusha_bulletin_board.c,v 1.31 2004/01/15 02:05:08 fuyu Exp $
  */
 
 #include "config.h"
@@ -40,6 +40,9 @@
 #include <libxml/parser.h>
 
 #include <fcntl.h>
+
+#include <pthread.h>
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -225,6 +228,17 @@ ochusha_bulletin_board_init(OchushaBulletinBoard *board)
   board->dropped_list = NULL;
 
   board->bbs_type = OCHUSHA_BBS_TYPE_UNKNOWN;
+
+  if (pthread_mutex_init(&board->thread_list_lock, NULL) != 0)
+    {
+      fprintf(stderr, "Couldn't initialize a mutex.\n");
+      abort();
+    }
+  if (pthread_cond_init(&board->thread_list_cond, NULL) != 0)
+    {
+      fprintf(stderr, "Couldn't initialize a condvar.\n");
+      abort();
+    }
 }
 
 
@@ -284,6 +298,17 @@ ochusha_bulletin_board_finalize(GObject *object)
       board->dropped_list = NULL;
     }
 
+  if (pthread_mutex_destroy(&board->thread_list_lock) != 0)
+    {
+      fprintf(stderr, "Couldn't destroy a mutex.\n");
+      abort();
+    }
+  if (pthread_cond_destroy(&board->thread_list_cond) != 0)
+    {
+      fprintf(stderr, "Couldn't destroy a condvar.\n");
+      abort();
+    }
+
   if (G_OBJECT_CLASS(parent_class)->finalize)
     (*G_OBJECT_CLASS(parent_class)->finalize)(object);
 }
@@ -504,6 +529,50 @@ ochusha_bulletin_board_get_id(OchushaBulletinBoard *board)
 }
 
 
+void
+ochusha_bulletin_board_lock_thread_list(OchushaBulletinBoard *board)
+{
+  g_assert(OCHUSHA_IS_BULLETIN_BOARD(board));
+
+  if (pthread_mutex_lock(&board->thread_list_lock) != 0)
+    {
+      fprintf(stderr, "Couldn't lock a mutex.\n");
+      abort();
+    }
+  while (board->is_busy)
+    {
+      fprintf(stderr, "board->is_busy.\n");
+      if (pthread_cond_wait(&board->thread_list_cond,
+			    &board->thread_list_lock) != 0)
+	{
+	  fprintf(stderr, "Couldn't wait a condition.\n");
+	  abort();
+	}
+    }
+  board->is_busy = TRUE;
+}
+
+
+void
+ochusha_bulletin_board_unlock_thread_list(OchushaBulletinBoard *board)
+{
+  g_assert(OCHUSHA_IS_BULLETIN_BOARD(board));
+  g_assert(board->is_busy);	/* ٤åġ */
+
+  board->is_busy = FALSE;
+  if (pthread_cond_signal(&board->thread_list_cond) != 0)
+    {
+      fprintf(stderr, "Couldn't signal a condition.\n");
+      abort();
+    }
+  if (pthread_mutex_unlock(&board->thread_list_lock) != 0)
+    {
+      fprintf(stderr, "Couldn't unlock a mutex.\n");
+      abort();
+    }
+}
+
+
 OchushaBBSThread *
 ochusha_bulletin_board_bbs_thread_new(OchushaBulletinBoard *board,
 				      const char *id, const gchar *title)
@@ -523,7 +592,7 @@ ochusha_bulletin_board_bbs_thread_new(OchushaBulletinBoard *board,
 
   thread = (*klass->thread_new)(board, id, title);
 
-  OCHU_OBJECT_REF(G_OBJECT(thread));
+  OCHU_OBJECT_REF(thread);
   if (thread != NULL)
     g_hash_table_insert(board->thread_table, thread->id, thread);
     
@@ -600,13 +669,18 @@ ochusha_bulletin_board_refresh_threadlist(OchushaBulletinBoard *board,
 					  gpointer callback_data)
 {
   OchushaBulletinBoardClass *klass;
+  gboolean result;
   g_return_val_if_fail(OCHUSHA_IS_BULLETIN_BOARD(board) && buffer != NULL,
 		       NULL);
 
   klass = OCHUSHA_BULLETIN_BOARD_GET_CLASS(board);
   g_return_val_if_fail(klass->get_threadlist_source != NULL, NULL);
 
-  return (*klass->refresh_threadlist)(board, buffer, cb, callback_data);
+  ochusha_bulletin_board_lock_thread_list(board);
+  result = (*klass->refresh_threadlist)(board, buffer, cb, callback_data);
+  ochusha_bulletin_board_unlock_thread_list(board);
+
+  return result;
 }
 
 
@@ -1202,6 +1276,8 @@ ochusha_bulletin_board_read_threadlist_xml(OchushaBulletinBoard *board,
 #endif
   sax_handler.characters = charactersHandler;
 
+  ochusha_bulletin_board_lock_thread_list(board);
+
   xmlSAXUserParseFile(&sax_handler, &context, threadlist_xml);
 
   cleanup_sax_context(&context);
@@ -1214,6 +1290,8 @@ ochusha_bulletin_board_read_threadlist_xml(OchushaBulletinBoard *board,
 	(*board_class->recover_threadlist)(board, config);
     }
 
+  ochusha_bulletin_board_unlock_thread_list(board);
+
   if (context.state == SAX_ACCEPTED)
     {
       G_FREE(threadlist_xml);
