@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003 The Ochusha Project.
+ * Copyright (c) 2003-2004 The Ochusha Project.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_bulletin_board.h,v 1.18 2004/01/11 22:29:14 fuyu Exp $
+ * $Id: ochusha_bulletin_board.h,v 1.20 2004/01/15 02:12:14 fuyu Exp $
  */
 
 #ifndef _OCHUSHA_BULLETIN_BOARD_H_
@@ -36,6 +36,8 @@
 #include <glib-object.h>
 #include <glib.h>
 
+#include <pthread.h>
+
 #include <zlib.h>
 
 
@@ -95,6 +97,10 @@ struct _OchushaBulletinBoard
 
   gboolean hidden;		/* İ鱣 */
   gboolean killed;		/* İ¸˼ΤƤ */
+
+  volatile gboolean is_busy;
+  pthread_mutex_t thread_list_lock;
+  pthread_cond_t thread_list_cond;
 };
 
 
@@ -174,6 +180,9 @@ void ochusha_bulletin_board_set_bbs_type(OchushaBulletinBoard *board,
 					 int bbs_type);
 int ochusha_bulletin_board_get_bbs_type(OchushaBulletinBoard *board);
 
+void ochusha_bulletin_board_lock_thread_list(OchushaBulletinBoard *board);
+void ochusha_bulletin_board_unlock_thread_list(OchushaBulletinBoard *board);
+
 OchushaBBSThread *ochusha_bulletin_board_bbs_thread_new(
 					OchushaBulletinBoard *board,
 					const char *id, const gchar *title);
