@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003 The Ochusha Project.
+ * Copyright (c) 2003-2004 The Ochusha Project.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_network_broker.c,v 1.45 2004/01/13 22:58:30 fuyu Exp $
+ * $Id: ochusha_network_broker.c,v 1.46 2004/01/14 10:11:06 fuyu Exp $
  */
 
 #include "config.h"
@@ -435,8 +435,6 @@ get_content_encoding_type(ghttp_request *request)
  */
 typedef struct _NetworkBrokerJobArgs
 {
-  OchushaNetworkBroker *broker;
-
   int length;
   char *url;
   char *if_modified_since;
@@ -633,7 +631,7 @@ timeout_cb(WorkerSyncObject *sync_object)
 	  sync_object, G_OBJECT(sync_object)->ref_count);
 #endif
 
-  OCHU_OBJECT_UNREF(G_OBJECT(sync_object));
+  OCHU_OBJECT_UNREF(sync_object);
 
   return FALSE;
 }
@@ -664,7 +662,7 @@ poll_cb(GIOChannel *channel, GIOCondition condition,
   fprintf(stderr, "leaving: poll_cb(%p): count=%d\n",
 	  sync_object, G_OBJECT(sync_object)->ref_count);
 #endif
-  OCHU_OBJECT_UNREF(G_OBJECT(sync_object));
+  OCHU_OBJECT_UNREF(sync_object);
 
   return FALSE;
 }
@@ -677,7 +675,7 @@ register_polling_function_for_read(WorkerSyncObject *sync_object, int fd)
   if (sync_object->poll_id == 0)
     {
       GIOChannel *channel = g_io_channel_unix_new(fd);
-      OCHU_OBJECT_REF(G_OBJECT(sync_object));
+      OCHU_OBJECT_REF(sync_object);
       sync_object->poll_id
 	= g_io_add_watch_full(channel,
 			      G_PRIORITY_DEFAULT,
@@ -1213,9 +1211,6 @@ destruct_job_args(NetworkBrokerJobArgs *job_args)
   if (job_args == NULL)
     return;
 
-  if (job_args->broker != NULL)
-    OCHU_OBJECT_UNREF(G_OBJECT(job_args->broker));
-
   if (job_args->url != NULL)
     G_FREE(job_args->url);
 
@@ -1261,7 +1256,7 @@ ochusha_network_broker_worker_sync_object_free(WorkerSyncObject *sync_object)
 	  sync_object, G_OBJECT(sync_object)->ref_count);
 #endif
 
-  OCHU_OBJECT_UNREF(G_OBJECT(sync_object));
+  OCHU_OBJECT_UNREF(sync_object);
 }
 
 
@@ -1280,6 +1275,8 @@ ochusha_network_broker_employ_worker_thread(OchushaNetworkBroker *broker,
   if (buffer == NULL || !ochusha_async_buffer_reset(buffer))
     buffer = ochusha_async_buffer_new(NULL, 0, NULL);
 
+  g_object_set_qdata(G_OBJECT(buffer), broker_id, broker);
+
   g_object_set_qdata_full(G_OBJECT(buffer), worker_sync_object_id, sync_object,
 		(GDestroyNotify)ochusha_network_broker_worker_sync_object_free);
   g_object_set_qdata_full(G_OBJECT(buffer), broker_buffer_status_id, status,
@@ -1298,7 +1295,6 @@ ochusha_network_broker_employ_worker_thread(OchushaNetworkBroker *broker,
   args = G_NEW0(NetworkBrokerJobArgs, 1);
   args->length = buffer->length;
   buffer->length = 0;
-  args->broker = broker;
   args->url = G_STRDUP(url);
   if (if_modified_since != NULL
       && (args->length > 0
@@ -1314,9 +1310,8 @@ ochusha_network_broker_employ_worker_thread(OchushaNetworkBroker *broker,
   job->args = buffer;
 
   /* åɤΤ˳ */
-  OCHU_OBJECT_REF(G_OBJECT(sync_object));
-  OCHU_OBJECT_REF(G_OBJECT(broker));
-  OCHU_OBJECT_REF(G_OBJECT(buffer));
+  OCHU_OBJECT_REF(sync_object);
+  OCHU_OBJECT_REF(buffer);
 
   commit_job(job);
 
@@ -1328,9 +1323,9 @@ static void
 try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 {
   gboolean signal_result;
-  NetworkBrokerJobArgs *args = g_object_get_qdata(G_OBJECT(buffer),
-						  broker_job_args_id);
-  OchushaNetworkBroker *broker;
+  NetworkBrokerJobArgs *args;
+  OchushaNetworkBroker *broker = g_object_get_qdata(G_OBJECT(buffer),
+						    broker_id);
   WorkerSyncObject *sync_object = g_object_get_qdata(G_OBJECT(buffer),
 						     worker_sync_object_id);
   OchushaNetworkBrokerBufferStatus *status
@@ -1343,48 +1338,67 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
   const char *error_message = NULL;
   gchar message[LOG_MESSAGE_SIZE];
 
-  if (args == NULL || !ochusha_async_buffer_active_ref(buffer))
+  if (!ochusha_async_buffer_active_ref(buffer))
     {
+      args = g_object_get_qdata(G_OBJECT(buffer), broker_job_args_id);
       if (args != NULL)
-	{
-	  broker = args->broker;
-	  url = args->url;
-	}
+	url = args->url;
       else
-	{
-	  broker = NULL;
-	  url = NULL;
-	}
+	url = NULL;
 
       status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_ACCESS_FAILED;
 
-      if (broker != NULL)
-	{
-	  g_signal_emit(G_OBJECT(broker),
-			broker_signals[ACCESS_TERMINATED_SIGNAL],
-			0,
-			buffer,
-			&signal_result);
+      g_signal_emit(G_OBJECT(broker),
+		    broker_signals[ACCESS_TERMINATED_SIGNAL],
+		    0,
+		    buffer,
+		    &signal_result);
 
-	  g_signal_emit(G_OBJECT(broker),
-			broker_signals[ACCESS_FAILED_SIGNAL],
-			0,
-			buffer,
-			OCHUSHA_NETWORK_BROKER_FAILURE_REASON_ACCESS_TERMINATED,
-			_("Access terminated."),
-			&signal_result);
-	}
+      g_signal_emit(G_OBJECT(broker),
+		    broker_signals[ACCESS_FAILED_SIGNAL],
+		    0,
+		    buffer,
+		    OCHUSHA_NETWORK_BROKER_FAILURE_REASON_ACCESS_TERMINATED,
+		    _("Access terminated."),
+		    &signal_result);
 
       ochusha_async_buffer_fix(buffer);
       g_object_set_qdata(G_OBJECT(buffer), broker_job_args_id, NULL);
-      OCHU_OBJECT_UNREF(G_OBJECT(buffer));
+      OCHU_OBJECT_UNREF(buffer);
 
-      if (broker != NULL)
-	{
-	  snprintf(message, LOG_MESSAGE_SIZE,
-		   _("Access terminated: %s\n"), url);
-	  ochusha_network_broker_output_log(broker, message);
-	}
+      snprintf(message, LOG_MESSAGE_SIZE,
+	       _("Access terminated: %s\n"), url);
+      ochusha_network_broker_output_log(broker, message);
+
+      LOCK_WORKER_SYNC_OBJECT(sync_object);
+      sync_object->finished = TRUE;
+      UNLOCK_WORKER_SYNC_OBJECT(sync_object);
+
+      return;
+    }
+
+  args = g_object_get_qdata(G_OBJECT(buffer), broker_job_args_id);
+  if (args == NULL)
+    {
+      status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_ACCESS_FAILED;
+
+      g_signal_emit(G_OBJECT(broker),
+		    broker_signals[ACCESS_TERMINATED_SIGNAL],
+		    0,
+		    buffer,
+		    &signal_result);
+
+      g_signal_emit(G_OBJECT(broker),
+		    broker_signals[ACCESS_FAILED_SIGNAL],
+		    0,
+		    buffer,
+		    OCHUSHA_NETWORK_BROKER_FAILURE_REASON_ACCESS_TERMINATED,
+		    _("Access terminated."),
+		    &signal_result);
+
+      ochusha_async_buffer_fix(buffer);
+      g_object_set_qdata(G_OBJECT(buffer), broker_job_args_id, NULL);
+      OCHU_OBJECT_UNREF(buffer);
 
       LOCK_WORKER_SYNC_OBJECT(sync_object);
       sync_object->finished = TRUE;
@@ -1393,7 +1407,6 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
       return;
     }
 
-  broker = args->broker;
   url = args->url;
 
   if (args->length == 0)
@@ -1420,7 +1433,7 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 
 	  ochusha_async_buffer_fix(buffer);
 	  g_object_set_qdata(G_OBJECT(buffer), broker_job_args_id, NULL);
-	  OCHU_OBJECT_UNREF(G_OBJECT(buffer));
+	  OCHU_OBJECT_UNREF(buffer);
 
 	  snprintf(message, LOG_MESSAGE_SIZE,
 		   _("Access terminated: %s\n"), url);
@@ -1901,7 +1914,7 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
   ochusha_async_buffer_fix(buffer);
   ochusha_async_buffer_active_unref(buffer);
   g_object_set_qdata(G_OBJECT(buffer), broker_job_args_id, NULL);
-  OCHU_OBJECT_UNREF(G_OBJECT(buffer));
+  OCHU_OBJECT_UNREF(buffer);
   return;
 
  error_exit:
@@ -1979,7 +1992,7 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
   ochusha_async_buffer_fix(buffer);
   ochusha_async_buffer_active_unref(buffer);
   g_object_set_qdata(G_OBJECT(buffer), broker_job_args_id, NULL);
-  OCHU_OBJECT_UNREF(G_OBJECT(buffer));
+  OCHU_OBJECT_UNREF(buffer);
 }
 
 
@@ -1987,9 +2000,9 @@ static void
 refresh_cache_after_read(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 {
   gboolean signal_result;
-  NetworkBrokerJobArgs *args = g_object_get_qdata(G_OBJECT(buffer),
-					      broker_job_args_id);
-  OchushaNetworkBroker *broker;
+  NetworkBrokerJobArgs *args;
+  OchushaNetworkBroker *broker = g_object_get_qdata(G_OBJECT(buffer),
+						    broker_id);
   OchushaNetworkBrokerBufferStatus *status
     = g_object_get_qdata(G_OBJECT(buffer), broker_buffer_status_id);
   gchar message[LOG_MESSAGE_SIZE];
@@ -1998,22 +2011,28 @@ refresh_cache_after_read(WorkerThread *employee, OchushaAsyncBuffer *buffer)
   fprintf(stderr, "refresh_cache_after_read() thread is invoked.\n");
 #endif
 
-  if (args == NULL || !ochusha_async_buffer_active_ref(buffer))
+  if (!ochusha_async_buffer_active_ref(buffer))
     {
       status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_ACCESS_FAILED;
-      if (args != NULL)
-	{
-	  broker = args->broker;
-	  g_signal_emit(G_OBJECT(broker),
-			broker_signals[ACCESS_TERMINATED_SIGNAL],
-			0,
-			buffer,
-			&signal_result);
-	}
+      g_signal_emit(G_OBJECT(broker),
+		    broker_signals[ACCESS_TERMINATED_SIGNAL],
+		    0,
+		    buffer,
+		    &signal_result);
       goto finish_refresh_cache_after_read;
     }
 
-  broker = args->broker;
+  args = g_object_get_qdata(G_OBJECT(buffer), broker_job_args_id);
+  if (args == NULL)
+    {
+      status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_ACCESS_FAILED;
+      g_signal_emit(G_OBJECT(broker),
+		    broker_signals[ACCESS_TERMINATED_SIGNAL],
+		    0,
+		    buffer,
+		    &signal_result);
+      goto finish_refresh_cache_after_read;
+    }
 
   status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_DIRECT_ACCESS;
 
@@ -2100,7 +2119,7 @@ refresh_cache_after_read(WorkerThread *employee, OchushaAsyncBuffer *buffer)
  finish_refresh_cache_after_read:
   ochusha_async_buffer_fix(buffer);
   g_object_set_qdata(G_OBJECT(buffer), broker_job_args_id, NULL);
-  OCHU_OBJECT_UNREF(G_OBJECT(buffer));
+  OCHU_OBJECT_UNREF(buffer);
 }
 
 
@@ -2108,9 +2127,9 @@ static void
 force_read(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 {
   gboolean signal_result;
-  NetworkBrokerJobArgs *args = g_object_get_qdata(G_OBJECT(buffer),
-					      broker_job_args_id);
-  OchushaNetworkBroker *broker;
+  NetworkBrokerJobArgs *args;
+  OchushaNetworkBroker *broker = g_object_get_qdata(G_OBJECT(buffer),
+						    broker_id);
   OchushaNetworkBrokerBufferStatus *status
     = g_object_get_qdata(G_OBJECT(buffer), broker_buffer_status_id);
 
@@ -2118,22 +2137,28 @@ force_read(WorkerThread *employee, OchushaAsyncBuffer *buffer)
   fprintf(stderr, "force_read() thread is invoked.\n");
 #endif
 
-  if (args == NULL || !ochusha_async_buffer_active_ref(buffer))
+  if (!ochusha_async_buffer_active_ref(buffer))
     {
       status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_ACCESS_FAILED;
-      if (args != NULL)
-	{
-	  broker = args->broker;
-	  g_signal_emit(G_OBJECT(broker),
-			broker_signals[ACCESS_TERMINATED_SIGNAL],
-			0,
-			buffer,
-			&signal_result);
-	}
+      g_signal_emit(G_OBJECT(broker),
+		    broker_signals[ACCESS_TERMINATED_SIGNAL],
+		    0,
+		    buffer,
+		    &signal_result);
       goto finish_force_read;
     }
 
-  broker = args->broker;
+  args = g_object_get_qdata(G_OBJECT(buffer), broker_job_args_id);
+  if (args == NULL)
+    {
+      status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_ACCESS_FAILED;
+      g_signal_emit(G_OBJECT(broker),
+		    broker_signals[ACCESS_TERMINATED_SIGNAL],
+		    0,
+		    buffer,
+		    &signal_result);
+      goto finish_force_read;
+    }
 
   status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_DIRECT_ACCESS;
 
@@ -2165,7 +2190,7 @@ force_read(WorkerThread *employee, OchushaAsyncBuffer *buffer)
  finish_force_read:
   ochusha_async_buffer_fix(buffer);
   g_object_set_qdata(G_OBJECT(buffer), broker_job_args_id, NULL);
-  OCHU_OBJECT_UNREF(G_OBJECT(buffer));
+  OCHU_OBJECT_UNREF(buffer);
 }
 
 
@@ -2214,8 +2239,8 @@ background_read_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 
  finish_job:
   ochusha_async_buffer_fix(buffer);
-  OCHU_OBJECT_UNREF(G_OBJECT(buffer));
-  OCHU_OBJECT_UNREF(G_OBJECT(broker));
+  OCHU_OBJECT_UNREF(buffer);
+  OCHU_OBJECT_UNREF(broker);
 }
 
 
@@ -2303,8 +2328,8 @@ ochusha_network_broker_read_from_url(OchushaNetworkBroker *broker,
 	  job->job = (JobFunc *)background_read_cache;
 	  job->args = buffer;
 
-	  OCHU_OBJECT_REF(G_OBJECT(broker));
-	  OCHU_OBJECT_REF(G_OBJECT(buffer));
+	  OCHU_OBJECT_REF(broker);
+	  OCHU_OBJECT_REF(buffer);
 
 	  commit_job(job);
 
@@ -2603,9 +2628,9 @@ static void
 force_read_synchronously(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 {
   gboolean signal_result;
-  NetworkBrokerJobArgs *args = g_object_get_qdata(G_OBJECT(buffer),
-					      broker_job_args_id);
-  OchushaNetworkBroker *broker;
+  NetworkBrokerJobArgs *args;
+  OchushaNetworkBroker *broker = g_object_get_qdata(G_OBJECT(buffer),
+						    broker_id);
   OchushaNetworkBrokerBufferStatus *status
     = g_object_get_qdata(G_OBJECT(buffer), broker_buffer_status_id);
 
@@ -2616,18 +2641,25 @@ force_read_synchronously(WorkerThread *employee, OchushaAsyncBuffer *buffer)
   if (!ochusha_async_buffer_active_ref(buffer))
     {
       status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_ACCESS_FAILED;
-      if (args != NULL)
-	{
-	  broker = args->broker;
-	  g_signal_emit(G_OBJECT(broker),
-			broker_signals[ACCESS_TERMINATED_SIGNAL],
-			0,
-			buffer,
-			&signal_result);
-	}
+      g_signal_emit(G_OBJECT(broker),
+		    broker_signals[ACCESS_TERMINATED_SIGNAL],
+		    0,
+		    buffer,
+		    &signal_result);
+      goto finish_force_read;
+    }
+
+  args = g_object_get_qdata(G_OBJECT(buffer), broker_job_args_id);
+  if (args == NULL)
+    {
+      status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_ACCESS_FAILED;
+      g_signal_emit(G_OBJECT(broker),
+		    broker_signals[ACCESS_TERMINATED_SIGNAL],
+		    0,
+		    buffer,
+		    &signal_result);
       goto finish_force_read;
     }
-  broker = args->broker;
 
   status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_DIRECT_ACCESS;
 
@@ -2659,7 +2691,7 @@ force_read_synchronously(WorkerThread *employee, OchushaAsyncBuffer *buffer)
  finish_force_read:
   ochusha_async_buffer_fix(buffer);
   g_object_set_qdata(G_OBJECT(buffer), broker_job_args_id, NULL);
-  OCHU_OBJECT_UNREF(G_OBJECT(buffer));
+  OCHU_OBJECT_UNREF(buffer);
 }
 
 
