@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_thread_2ch.c,v 1.58 2004/01/15 02:08:58 fuyu Exp $
+ * $Id: ochusha_thread_2ch.c,v 1.59 2004/01/16 08:00:01 fuyu Exp $
  */
 
 #include "config.h"
@@ -49,7 +49,7 @@
 #define DEBUG_DAT	0
 #define DEBUG_DAT_MOST	0
 
-#define ANALYSIS_INTERVAL	20
+#define ANALYSIS_INTERVAL	100
 
 
 static void ochusha_thread_2ch_class_init(OchushaThread2chClass *klass);
@@ -138,6 +138,8 @@ ochusha_thread_2ch_get_type(void)
 
 
 static OchushaBBSThreadClass *parent_class = NULL;
+static GQuark idle_checker_id;
+
 
 static void
 ochusha_thread_2ch_class_init(OchushaThread2chClass *klass)
@@ -176,6 +178,9 @@ ochusha_thread_2ch_class_init(OchushaThread2chClass *klass)
 
   klass->get_base_path = NULL;
   klass->make_post_response_message = NULL;
+
+  idle_checker_id
+    = g_quark_from_static_string("OchushaThread2ch::IdleChecker");
 }
 
 
@@ -731,7 +736,7 @@ extract_2ch_response(char *start_pos, char *eol_pos,
 static gboolean
 advance_parsing(gpointer data)
 {
-  g_object_set_data(G_OBJECT(data), "checker", NULL);
+  g_object_set_qdata(G_OBJECT(data), idle_checker_id, NULL);
   ochusha_async_buffer_broadcast((OchushaAsyncBuffer *)data);
   OCHU_OBJECT_UNREF(data);
   return FALSE;
@@ -999,47 +1004,32 @@ ochusha_thread_2ch_parse_responses(OchushaBBSThread *thread,
 	    if (no_wait)
 	      goto terminated;	/* ξ */
 
-	    if (ochusha_async_buffer_is_busy(buffer))
-	      do
-		{
-		  if (!ochusha_async_buffer_wait(buffer))
-		    {
+	    do
+	      {
+		if (!ochusha_async_buffer_wait(buffer))
+		  {
 #if DEBUG_ASYNC_BUFFER_MOST
-		      fprintf(stderr, "ochusha_thread_2ch_parse_responses(): buffer has been terminated.\n");
+		    fprintf(stderr, "ochusha_thread_2ch_parse_responses(): buffer has been terminated.\n");
 #endif
-		      goto terminated;
-		    }
-		} while (buffer->length == old_len && !buffer->fixed);
+		    goto terminated;
+		  }
+	      } while (buffer->length == old_len && !buffer->fixed);
 	  }
 	else
 	  {
-#if 0
-	    if (ochusha_async_buffer_is_busy(buffer))
+	    /*
+	     * GUIΥ쥹ݥ󥹤βܻؤ¸
+	     */
+	    OCHU_OBJECT_REF(buffer);
+	    g_object_set_qdata(G_OBJECT(buffer), idle_checker_id, buffer);
+	    g_idle_add_full(G_PRIORITY_HIGH_IDLE + 15,
+			    advance_parsing, buffer, NULL);
+	    do
 	      {
-		/*
-		 * buffer˥륹åɤsuspend/terminate
-		 * ȿ뤿˥å
-		 */
 		if (!ochusha_async_buffer_wait(buffer))
 		  goto terminated;
-	      }
-	    else
-#endif
-	      {
-		/*
-		 * GUIΥ쥹ݥ󥹤βܻؤ¸
-		 */
-		OCHU_OBJECT_REF(buffer);
-		g_object_set_data(G_OBJECT(buffer), "checker", buffer);
-		g_idle_add_full(G_PRIORITY_HIGH_IDLE + 15,
-				advance_parsing, buffer, NULL);
-		do
-		  {
-		    if (!ochusha_async_buffer_wait(buffer))
-		      goto terminated;
-		  } while (g_object_get_data(G_OBJECT(buffer), "checker")
-			   != NULL);
-	      }
+	      } while (g_object_get_qdata(G_OBJECT(buffer), idle_checker_id)
+		       != NULL);
 	    interval = ANALYSIS_INTERVAL;
 	  }
 
