@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_thread_jbbs.c,v 1.26 2004/01/14 09:59:48 fuyu Exp $
+ * $Id: ochusha_thread_jbbs.c,v 1.28 2004/01/16 08:00:01 fuyu Exp $
  */
 
 #include "config.h"
@@ -59,7 +59,7 @@
 #define DEBUG_DAT	0
 #define DEBUG_DAT_MOST	0
 
-#define ANALYSIS_INTERVAL	20
+#define ANALYSIS_INTERVAL	100
 
 
 static void ochusha_thread_jbbs_class_init(OchushaThreadJBBSClass *klass);
@@ -142,6 +142,8 @@ ochusha_thread_jbbs_get_type(void)
 
 
 static OchushaThread2chClass *parent_class = NULL;
+static GQuark idle_checker_id;
+
 
 static void
 ochusha_thread_jbbs_class_init(OchushaThreadJBBSClass *klass)
@@ -171,6 +173,9 @@ ochusha_thread_jbbs_class_init(OchushaThreadJBBSClass *klass)
   class_2ch->get_base_path = ochusha_thread_jbbs_get_base_path;
   class_2ch->make_post_response_message
     = ochusha_thread_jbbs_make_post_response_message;
+
+  idle_checker_id
+    = g_quark_from_static_string("OchushaThreadJBBS::IdleChecker");
 }
 
 
@@ -440,6 +445,7 @@ extract_jbbs_response(char *start_pos, char *eol_pos,
 static gboolean
 advance_parsing(gpointer data)
 {
+  g_object_set_qdata(G_OBJECT(data), idle_checker_id, NULL);
   ochusha_async_buffer_broadcast((OchushaAsyncBuffer *)data);
   OCHU_OBJECT_UNREF(data);
   return FALSE;
@@ -682,42 +688,34 @@ ochusha_thread_jbbs_parse_responses(OchushaBBSThread *thread,
 	    if (no_wait)
 	      goto terminated;	/* ξ */
 
-	    if (ochusha_async_buffer_is_busy(buffer))
-	      do
-		{
-		  if (!ochusha_async_buffer_wait(buffer))
-		    {
+	    do
+	      {
+		if (!ochusha_async_buffer_wait(buffer))
+		  {
 #if DEBUG_ASYNC_BUFFER_MOST
-		      fprintf(stderr, "ochusha_thread_jbbs_parse_responses(): buffer has been terminated.\n");
+		    fprintf(stderr, "ochusha_thread_jbbs_parse_responses(): buffer has been terminated.\n");
 #endif
-		      goto terminated;
-		    }
-		} while (buffer->length == old_len && !buffer->fixed);
+		    goto terminated;
+		  }
+	      } while (buffer->length == old_len && !buffer->fixed);
 	  }
 	else
 	  {
-	    if (ochusha_async_buffer_is_busy(buffer))
+	    /*
+	     * GUIΥ쥹ݥ󥹤βܻؤ¸
+	     */
+	    OCHU_OBJECT_REF(buffer);
+	    g_object_set_qdata(G_OBJECT(buffer), idle_checker_id, buffer);
+	    g_idle_add_full(G_PRIORITY_HIGH_IDLE + 15,
+			    advance_parsing, buffer, NULL);
+	    do
 	      {
-		/*
-		 * buffer˥륹åɤsuspend/terminate
-		 * ȿ뤿˥å
-		 */
 		if (!ochusha_async_buffer_wait(buffer))
 		  goto terminated;
-	      }
-	    else
-	      {
-		/*
-		 * GUIΥ쥹ݥ󥹤βܻؤ¸
-		 */
-		OCHU_OBJECT_REF(buffer);
-		g_idle_add_full(G_PRIORITY_HIGH_IDLE + 15,
-				advance_parsing, buffer, NULL);
-		if (!ochusha_async_buffer_wait(buffer))
-		  goto terminated;
-	      }
+	      } while (g_object_get_qdata(G_OBJECT(buffer), idle_checker_id)
+		       != NULL);
+	    interval = ANALYSIS_INTERVAL;
 	  }
-	interval = ANALYSIS_INTERVAL;
 
 	if (status->state
 	    == OCHUSHA_NETWORK_BROKER_BUFFER_STATE_CACHE_IS_DIRTY)
@@ -1185,7 +1183,7 @@ convert_html_to_dat(WorkerThread *employee, HTML2DATConverterJobArgs *args)
 						   rest_of_data)) != NULL)
 	{
 	  int line_size = eol_pos - html_buffer_pos;
-	  char *html_line_buffer = G_STRNDUP(html_buffer_pos, line_size);
+	  char *html_line_buffer = g_strndup(html_buffer_pos, line_size);
 	  char *html_line_buffer_tail = html_line_buffer + line_size;
 	  line_size++;
 
@@ -1213,7 +1211,7 @@ convert_html_to_dat(WorkerThread *employee, HTML2DATConverterJobArgs *args)
 	    {
 	      char *title_head = html_line_buffer + match_region->beg[1];
 	      int title_len = match_region->end[1] - match_region->beg[1];
-	      title = G_STRNDUP(title_head, title_len);
+	      title = g_strndup(title_head, title_len);
 #if DEBUG_HTML_TO_DAT_CONVERTER_MOST
 	      fprintf(stderr, "title: %s\n", title);
 #endif
@@ -1414,7 +1412,7 @@ convert_html_to_dat(WorkerThread *employee, HTML2DATConverterJobArgs *args)
 
 	      if (title != NULL)
 		{
-		  G_FREE(title);
+		  g_free(title);
 		  title = NULL;
 		}
 	    }
@@ -1426,7 +1424,7 @@ convert_html_to_dat(WorkerThread *employee, HTML2DATConverterJobArgs *args)
 		  fprintf(stderr, "IRREGULAR html_line: \"%s\"\n",
 			  html_line_buffer);
 #endif
-		  G_FREE(html_line_buffer);
+		  g_free(html_line_buffer);
 		  html_line_buffer = previous_line;
 		  previous_line = NULL;
 		}
@@ -1450,7 +1448,7 @@ convert_html_to_dat(WorkerThread *employee, HTML2DATConverterJobArgs *args)
 	    }
 
 	  if (html_line_buffer != NULL)
-	    G_FREE(html_line_buffer);
+	    g_free(html_line_buffer);
 
 	  html_buffer_pos = eol_pos + 1;
 	  rest_of_data -= line_size;
@@ -1469,7 +1467,7 @@ convert_html_to_dat(WorkerThread *employee, HTML2DATConverterJobArgs *args)
 
 	  if (previous_line != NULL)
 	    {
-	      G_FREE(previous_line);
+	      g_free(previous_line);
 	      previous_line = NULL;
 	    }
 	  break;
@@ -1486,7 +1484,7 @@ convert_html_to_dat(WorkerThread *employee, HTML2DATConverterJobArgs *args)
 				&signal_result);
 	  if (previous_line != NULL)
 	    {
-	      G_FREE(previous_line);
+	      g_free(previous_line);
 	      previous_line = NULL;
 	    }
 	  break;
