@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003 The Ochusha Project.
+ * Copyright (c) 2003-2004 The Ochusha Project.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: utils.c,v 1.19 2004/01/13 14:52:49 fuyu Exp $
+ * $Id: utils.c,v 1.23 2004/01/14 14:45:22 fuyu Exp $
  */
 
 #define OCHUSHA_UTILS_IMPLEMENTATION
@@ -504,6 +504,85 @@ ochusha_utils_get_utc_time(const char *date_string)
 }
 
 
+/*
+ * Ĺǽstrpbrkؿ
+ * stringĹlenʸcharset˴ޤޤǽʸؤΥݥ󥿤֤
+ * ʤNULL֤
+ */
+const char *
+mempbrk(const char *string, const char *charset, size_t len)
+{
+  while (len > 0)
+    {
+      const char *c = charset;
+      char s = *string;
+      while (*c)
+	{
+	  if (*c == s)
+	    return string;
+	  c++;
+	}
+      string++;
+      len--;
+    }
+  return NULL;
+}
+
+
+const char *
+reverse_strpbrk(const char *string, const char *charset)
+{
+  const gchar *s = string;
+  while (*s != '\0')
+    {
+      if (strchr(charset, *s) == NULL)
+	return s;
+      s++;
+    }
+
+  return NULL;
+}
+
+
+int
+parse_int(const gchar *start, size_t len)
+{
+  int result = 0;
+#if 1
+  const gchar *tmp_pos = start;
+  const gchar *tail_pos = (len > 0 ? start + len : NULL);
+  gunichar c;
+
+  do
+    {
+      c = g_utf8_get_char(tmp_pos);
+      if (c >= '0' && c <= '9')
+	result = result * 10 + c - '0';
+      else if (c >= 65296 && c <= 65305)
+	result = result * 10 + c - 65296;
+      else
+	break;
+      tmp_pos = g_utf8_find_next_char(tmp_pos, tail_pos);
+    } while (tmp_pos != NULL);
+
+  return result;
+#else
+  char tmp_buffer[128];
+
+  if (len > 0)
+    {
+      len = MIN(127, len);
+      memcpy(tmp_buffer, start, len);
+      tmp_buffer[len] = '\0';
+    }
+  else
+    g_strlcpy(tmp_buffer, start, 128);
+  sscanf(tmp_buffer, "%d", &result);
+  return result;
+#endif
+}
+
+
 char *
 ochusha_utils_find_cgi_query_key(const char *src, CGIQueryKeyTuple *result)
 {
@@ -1763,10 +1842,11 @@ init_refcount_trace(void)
 
 
 void
-trace_register_g_object(GObject *object, const char *file, int line)
+trace_register_g_object(gpointer pointer, const char *file, int line)
 {
   GSList *previous;
   char where[1024];
+  GObject *object = (GObject *)pointer;
   snprintf(where, 1024, "register_g_object(%p) at %s, line %d: ref_count=%d",
 	   object, file, line, object->ref_count);
   REFCOUNT_TRACE_LOCK
@@ -1790,10 +1870,11 @@ trace_register_g_object(GObject *object, const char *file, int line)
 
 
 void
-trace_g_object_ref(GObject *object, const char *file, int line)
+trace_g_object_ref(gpointer pointer, const char *file, int line)
 {
   GSList *history;
   char where[1024];
+  GObject *object = (GObject *)pointer;
   snprintf(where, 1024, "g_object_ref(%p) at %s, line %d: ref_count=%d",
 	   object, file, line, object->ref_count);
   REFCOUNT_TRACE_LOCK
@@ -1808,10 +1889,11 @@ trace_g_object_ref(GObject *object, const char *file, int line)
 
 
 void
-trace_g_object_unref(GObject *object, const char *file, int line)
+trace_g_object_unref(gpointer pointer, const char *file, int line)
 {
   GSList *history;
   char where[1024];
+  GObject *object = (GObject *)pointer;
   snprintf(where, 1024, "g_object_unref(%p) at %s, line %d: ref_count=%d",
 	   object, file, line, object->ref_count);
   REFCOUNT_TRACE_LOCK
@@ -1838,7 +1920,7 @@ print_history(const char *where)
 
 
 static void
-remove_consistent_object(gpointer object)
+remove_consistent_object(gpointer object, gpointer unused)
 {
   GSList *history = g_hash_table_lookup(refcount_trace, object);
 #if 0
@@ -1881,8 +1963,9 @@ output_warning(GObject *object)
 
 
 void
-ochu_object_ref(GObject *object)
+ochu_object_ref(gpointer pointer)
 {
+  GObject *object = (GObject *)pointer;
   if (object->ref_count >= 100)
     output_warning(object);
   OCHU_OBJECT_REF(object);
@@ -1890,8 +1973,9 @@ ochu_object_ref(GObject *object)
 
 
 void
-ochu_object_unref(GObject *object)
+ochu_object_unref(gpointer pointer)
 {
+  GObject *object = (GObject *)pointer;
   if (object->ref_count < 1 || object->ref_count >= 100)
     output_warning(object);
   OCHU_OBJECT_UNREF(object);
